package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.AllocationExpression;
import org.eclipse.jdt.internal.compiler.ast.CastExpression;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.ExpressionContext;
import org.eclipse.jdt.internal.compiler.ast.FakedTrackingVariable;
import org.eclipse.jdt.internal.compiler.ast.Invocation;
import org.eclipse.jdt.internal.compiler.ast.ParameterizedQualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.QualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.SingleTypeReference;
import org.eclipse.jdt.internal.compiler.ast.Statement;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeReference;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ImplicitNullAnnotationVerifier;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.LocalTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedGenericMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.PolyTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.Substitution;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;

public class QualifiedAllocationExpression extends AllocationExpression {
  public Expression enclosingInstance;
  
  public TypeDeclaration anonymousType;
  
  public QualifiedAllocationExpression() {}
  
  public QualifiedAllocationExpression(TypeDeclaration anonymousType) {
    this.anonymousType = anonymousType;
    anonymousType.allocation = this;
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    if (this.enclosingInstance != null) {
      flowInfo = this.enclosingInstance.analyseCode(currentScope, flowContext, flowInfo);
    } else if (this.binding != null && this.binding.declaringClass != null) {
      ReferenceBinding superclass = this.binding.declaringClass.superclass();
      if (superclass != null && superclass.isMemberType() && !superclass.isStatic())
        currentScope.tagAsAccessingEnclosingInstanceStateOf(superclass.enclosingType(), false); 
    } 
    checkCapturedLocalInitializationIfNecessary(
        (this.anonymousType == null) ? 
        (ReferenceBinding)this.binding.declaringClass.erasure() : 
        (ReferenceBinding)this.binding.declaringClass.superclass().erasure(), 
        currentScope, 
        flowInfo);
    if (this.arguments != null) {
      boolean analyseResources = (currentScope.compilerOptions()).analyseResourceLeaks;
      boolean hasResourceWrapperType = (analyseResources && 
        this.resolvedType instanceof ReferenceBinding && (
        (ReferenceBinding)this.resolvedType).hasTypeBit(4));
      for (int i = 0, count = this.arguments.length; i < count; i++) {
        flowInfo = this.arguments[i].analyseCode(currentScope, flowContext, flowInfo);
        if (analyseResources && !hasResourceWrapperType)
          flowInfo = FakedTrackingVariable.markPassedToOutside(currentScope, this.arguments[i], flowInfo, flowContext, false); 
        this.arguments[i].checkNPEbyUnboxing(currentScope, flowContext, flowInfo);
      } 
      analyseArguments(currentScope, flowContext, flowInfo, this.binding, this.arguments);
    } 
    if (this.anonymousType != null)
      flowInfo = this.anonymousType.analyseCode(currentScope, flowContext, flowInfo); 
    ReferenceBinding[] thrownExceptions;
    if ((thrownExceptions = this.binding.thrownExceptions).length != 0) {
      if ((this.bits & 0x10000) != 0 && this.genericTypeArguments == null)
        thrownExceptions = currentScope.environment().convertToRawTypes(this.binding.thrownExceptions, true, true); 
      flowContext.checkExceptionHandlers(
          (TypeBinding[])thrownExceptions, 
          (ASTNode)this, 
          (FlowInfo)flowInfo.unconditionalCopy(), 
          currentScope);
    } 
    if ((currentScope.compilerOptions()).analyseResourceLeaks && FakedTrackingVariable.isAnyCloseable(this.resolvedType))
      FakedTrackingVariable.analyseCloseableAllocation(currentScope, flowInfo, this); 
    manageEnclosingInstanceAccessIfNecessary(currentScope, flowInfo);
    manageSyntheticAccessIfNecessary(currentScope, flowInfo);
    flowContext.recordAbruptExit();
    return flowInfo;
  }
  
  public Expression enclosingInstance() {
    return this.enclosingInstance;
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
    cleanUpInferenceContexts();
    if (!valueRequired)
      currentScope.problemReporter().unusedObjectAllocation(this); 
    int pc = codeStream.position;
    MethodBinding codegenBinding = this.binding.original();
    ReferenceBinding allocatedType = codegenBinding.declaringClass;
    codeStream.new_(this.type, (TypeBinding)allocatedType);
    boolean isUnboxing = ((this.implicitConversion & 0x400) != 0);
    if (valueRequired || isUnboxing)
      codeStream.dup(); 
    if (this.type != null) {
      codeStream.recordPositionsFrom(pc, this.type.sourceStart);
    } else {
      codeStream.ldc(String.valueOf(this.enumConstant.name));
      codeStream.generateInlinedValue(this.enumConstant.binding.id);
    } 
    if (allocatedType.isNestedType())
      codeStream.generateSyntheticEnclosingInstanceValues(
          currentScope, 
          allocatedType, 
          enclosingInstance(), 
          (ASTNode)this); 
    generateArguments(this.binding, this.arguments, currentScope, codeStream);
    if (allocatedType.isNestedType())
      codeStream.generateSyntheticOuterArgumentValues(
          currentScope, 
          allocatedType, 
          (ASTNode)this); 
    if (this.syntheticAccessor == null) {
      codeStream.invoke((byte)-73, codegenBinding, null, this.typeArguments);
    } else {
      int i = 0;
      int max = this.syntheticAccessor.parameters.length - codegenBinding.parameters.length;
      for (; i < max; 
        i++)
        codeStream.aconst_null(); 
      codeStream.invoke((byte)-73, this.syntheticAccessor, null, this.typeArguments);
    } 
    if (valueRequired) {
      codeStream.generateImplicitConversion(this.implicitConversion);
    } else if (isUnboxing) {
      codeStream.generateImplicitConversion(this.implicitConversion);
      switch ((postConversionType((Scope)currentScope)).id) {
        case 7:
        case 8:
          codeStream.pop2();
          break;
        default:
          codeStream.pop();
          break;
      } 
    } 
    codeStream.recordPositionsFrom(pc, this.sourceStart);
    if (this.anonymousType != null)
      this.anonymousType.generateCode(currentScope, codeStream); 
  }
  
  public boolean isSuperAccess() {
    return (this.anonymousType != null);
  }
  
  public void manageEnclosingInstanceAccessIfNecessary(BlockScope currentScope, FlowInfo flowInfo) {
    if ((flowInfo.tagBits & 0x1) == 0) {
      ReferenceBinding allocatedTypeErasure = (ReferenceBinding)this.binding.declaringClass.erasure();
      if (allocatedTypeErasure.isNestedType() && (
        currentScope.enclosingSourceType().isLocalType() || currentScope.isLambdaSubscope()))
        if (allocatedTypeErasure.isLocalType()) {
          ((LocalTypeBinding)allocatedTypeErasure).addInnerEmulationDependent(currentScope, (this.enclosingInstance != null));
        } else {
          currentScope.propagateInnerEmulation(allocatedTypeErasure, (this.enclosingInstance != null));
        }  
    } 
  }
  
  public StringBuffer printExpression(int indent, StringBuffer output) {
    if (this.enclosingInstance != null)
      this.enclosingInstance.printExpression(0, output).append('.'); 
    super.printExpression(0, output);
    if (this.anonymousType != null)
      this.anonymousType.print(indent, output); 
    return output;
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    if (this.anonymousType == null && this.enclosingInstance == null)
      return super.resolveType(scope); 
    TypeBinding result = resolveTypeForQualifiedAllocationExpression(scope);
    if (result != null && !result.isPolyType() && this.binding != null) {
      CompilerOptions compilerOptions = scope.compilerOptions();
      if (compilerOptions.isAnnotationBasedNullAnalysisEnabled) {
        if ((this.binding.tagBits & 0x1000L) == 0L)
          (new ImplicitNullAnnotationVerifier(scope.environment(), compilerOptions.inheritNullAnnotations))
            .checkImplicitNullAnnotations(this.binding, null, false, (Scope)scope); 
        if (compilerOptions.sourceLevel >= 3407872L && 
          this.binding instanceof ParameterizedGenericMethodBinding && this.typeArguments != null) {
          TypeVariableBinding[] typeVariables = this.binding.original().typeVariables();
          for (int i = 0; i < this.typeArguments.length; i++)
            this.typeArguments[i].checkNullConstraints((Scope)scope, (Substitution)this.binding, (TypeBinding[])typeVariables, i); 
        } 
      } 
      if (compilerOptions.sourceLevel >= 3407872L && 
        this.binding.getTypeAnnotations() != Binding.NO_ANNOTATIONS)
        this.resolvedType = scope.environment().createAnnotatedType(this.resolvedType, this.binding.getTypeAnnotations()); 
    } 
    return result;
  }
  
  private TypeBinding resolveTypeForQualifiedAllocationExpression(BlockScope scope) {
    ReferenceBinding referenceBinding1;
    boolean isDiamond = (this.type != null && (this.type.bits & 0x80000) != 0);
    TypeBinding enclosingInstanceType = null;
    TypeBinding receiverType = null;
    long sourceLevel = (scope.compilerOptions()).sourceLevel;
    if (this.constant != Constant.NotAConstant) {
      ProblemReferenceBinding problemReferenceBinding;
      this.constant = Constant.NotAConstant;
      ReferenceBinding enclosingInstanceReference = null;
      boolean hasError = false;
      boolean enclosingInstanceContainsCast = false;
      if (this.enclosingInstance != null) {
        if (this.enclosingInstance instanceof CastExpression) {
          this.enclosingInstance.bits |= 0x20;
          enclosingInstanceContainsCast = true;
        } 
        if ((enclosingInstanceType = this.enclosingInstance.resolveType(scope)) == null) {
          hasError = true;
        } else if (enclosingInstanceType.isBaseType() || enclosingInstanceType.isArrayType()) {
          scope.problemReporter().illegalPrimitiveOrArrayTypeForEnclosingInstance(
              enclosingInstanceType, 
              (ASTNode)this.enclosingInstance);
          hasError = true;
        } else if (this.type instanceof QualifiedTypeReference) {
          scope.problemReporter().illegalUsageOfQualifiedTypeReference((QualifiedTypeReference)this.type);
          hasError = true;
        } else if (!(enclosingInstanceReference = (ReferenceBinding)enclosingInstanceType).canBeSeenBy((Scope)scope)) {
          problemReferenceBinding = new ProblemReferenceBinding(
              enclosingInstanceReference.compoundName, 
              enclosingInstanceReference, 
              2);
          scope.problemReporter().invalidType((ASTNode)this.enclosingInstance, (TypeBinding)problemReferenceBinding);
          hasError = true;
        } else {
          this.resolvedType = receiverType = ((SingleTypeReference)this.type).resolveTypeEnclosing(scope, (ReferenceBinding)problemReferenceBinding);
          checkIllegalNullAnnotation(scope, receiverType);
          if (receiverType != null && enclosingInstanceContainsCast)
            CastExpression.checkNeedForEnclosingInstanceCast(scope, this.enclosingInstance, (TypeBinding)problemReferenceBinding, receiverType); 
        } 
      } else if (this.type == null) {
        SourceTypeBinding sourceTypeBinding = scope.enclosingSourceType();
      } else {
        receiverType = this.type.resolveType(scope, true);
        checkIllegalNullAnnotation(scope, receiverType);
        if (receiverType != null && receiverType.isValidBinding() && 
          this.type instanceof ParameterizedQualifiedTypeReference) {
          ReferenceBinding currentType = (ReferenceBinding)receiverType;
          while ((currentType.modifiers & 0x8) == 0 && 
            !currentType.isRawType()) {
            if ((currentType = currentType.enclosingType()) == null) {
              ParameterizedQualifiedTypeReference qRef = (ParameterizedQualifiedTypeReference)this.type;
              for (int i = qRef.typeArguments.length - 2; i >= 0; i--) {
                if (qRef.typeArguments[i] != null) {
                  scope.problemReporter().illegalQualifiedParameterizedTypeAllocation(this.type, receiverType);
                  break;
                } 
              } 
              break;
            } 
          } 
        } 
      } 
      if (receiverType == null || !receiverType.isValidBinding())
        hasError = true; 
      if (this.typeArguments != null) {
        int length = this.typeArguments.length;
        this.argumentsHaveErrors = (sourceLevel < 3211264L);
        this.genericTypeArguments = new TypeBinding[length];
        int i;
        for (i = 0; i < length; i++) {
          TypeReference typeReference = this.typeArguments[i];
          this.genericTypeArguments[i] = typeReference.resolveType(scope, true);
          if (typeReference.resolveType(scope, true) == null)
            this.argumentsHaveErrors = true; 
          if (this.argumentsHaveErrors && typeReference instanceof org.eclipse.jdt.internal.compiler.ast.Wildcard)
            scope.problemReporter().illegalUsageOfWildcard(typeReference); 
        } 
        if (isDiamond) {
          scope.problemReporter().diamondNotWithExplicitTypeArguments(this.typeArguments);
          return null;
        } 
        if (this.argumentsHaveErrors) {
          if (this.arguments != null) {
            int max;
            for (i = 0, max = this.arguments.length; i < max; i++)
              this.arguments[i].resolveType(scope); 
          } 
          return null;
        } 
      } 
      this.argumentTypes = Binding.NO_PARAMETERS;
      if (this.arguments != null) {
        int length = this.arguments.length;
        this.argumentTypes = new TypeBinding[length];
        for (int i = 0; i < length; i++) {
          Expression argument = this.arguments[i];
          if (argument instanceof CastExpression) {
            argument.bits |= 0x20;
            this.argsContainCast = true;
          } 
          argument.setExpressionContext(ExpressionContext.INVOCATION_CONTEXT);
          this.argumentTypes[i] = argument.resolveType(scope);
          if (argument.resolveType(scope) == null)
            this.argumentsHaveErrors = hasError = true; 
        } 
      } 
      if (hasError) {
        if (isDiamond)
          return null; 
        if (receiverType instanceof ReferenceBinding) {
          ReferenceBinding referenceReceiver = (ReferenceBinding)receiverType;
          if (receiverType.isValidBinding()) {
            int length = (this.arguments == null) ? 0 : this.arguments.length;
            TypeBinding[] pseudoArgs = new TypeBinding[length];
            for (int i = length; --i >= 0;)
              pseudoArgs[i] = (this.argumentTypes[i] == null) ? (TypeBinding)TypeBinding.NULL : this.argumentTypes[i]; 
            this.binding = scope.findMethod(referenceReceiver, TypeConstants.INIT, pseudoArgs, (InvocationSite)this, false);
            if (this.binding != null && !this.binding.isValidBinding()) {
              MethodBinding closestMatch = ((ProblemMethodBinding)this.binding).closestMatch;
              if (closestMatch != null) {
                ParameterizedGenericMethodBinding parameterizedGenericMethodBinding;
                if ((closestMatch.original()).typeVariables != Binding.NO_TYPE_VARIABLES)
                  parameterizedGenericMethodBinding = scope.environment().createParameterizedGenericMethod(closestMatch.original(), null); 
                this.binding = (MethodBinding)parameterizedGenericMethodBinding;
                MethodBinding closestMatchOriginal = parameterizedGenericMethodBinding.original();
                if (closestMatchOriginal.isOrEnclosedByPrivateType() && !scope.isDefinedInMethod(closestMatchOriginal))
                  closestMatchOriginal.modifiers |= 0x8000000; 
              } 
            } 
          } 
          if (this.anonymousType != null) {
            scope.addAnonymousType(this.anonymousType, referenceReceiver);
            this.anonymousType.resolve(scope);
            return this.resolvedType = (TypeBinding)this.anonymousType.binding;
          } 
        } 
        return this.resolvedType = receiverType;
      } 
      if (this.anonymousType == null) {
        if (!receiverType.canBeInstantiated()) {
          scope.problemReporter().cannotInstantiate((Expression)this.type, receiverType);
          return this.resolvedType = receiverType;
        } 
      } else {
        ProblemReferenceBinding problemReferenceBinding1;
        if (isDiamond) {
          scope.problemReporter().diamondNotWithAnoymousClasses(this.type);
          return null;
        } 
        ReferenceBinding superType = (ReferenceBinding)receiverType;
        if (superType.isTypeVariable()) {
          problemReferenceBinding1 = new ProblemReferenceBinding(new char[][] { superType.sourceName() }, superType, 9);
          scope.problemReporter().invalidType((ASTNode)this, (TypeBinding)problemReferenceBinding1);
          return null;
        } 
        if (this.type != null && problemReferenceBinding1.isEnum()) {
          scope.problemReporter().cannotInstantiate((Expression)this.type, (TypeBinding)problemReferenceBinding1);
          return this.resolvedType = (TypeBinding)problemReferenceBinding1;
        } 
        ReferenceBinding anonymousSuperclass = problemReferenceBinding1.isInterface() ? scope.getJavaLangObject() : (ReferenceBinding)problemReferenceBinding1;
        scope.addAnonymousType(this.anonymousType, (ReferenceBinding)problemReferenceBinding1);
        this.anonymousType.resolve(scope);
        this.resolvedType = (TypeBinding)this.anonymousType.binding;
        if ((this.resolvedType.tagBits & 0x20000L) != 0L)
          return null; 
        MethodBinding inheritedBinding = findConstructorBinding(scope, (Invocation)this, anonymousSuperclass, this.argumentTypes);
        if (!inheritedBinding.isValidBinding()) {
          if (inheritedBinding.declaringClass == null)
            inheritedBinding.declaringClass = anonymousSuperclass; 
          if (this.type != null && !this.type.resolvedType.isValidBinding())
            return null; 
          scope.problemReporter().invalidConstructor((Statement)this, inheritedBinding);
          return this.resolvedType;
        } 
        if ((inheritedBinding.tagBits & 0x80L) != 0L)
          scope.problemReporter().missingTypeInConstructor((ASTNode)this, inheritedBinding); 
        if (this.enclosingInstance != null) {
          ReferenceBinding targetEnclosing = inheritedBinding.declaringClass.enclosingType();
          if (targetEnclosing == null) {
            scope.problemReporter().unnecessaryEnclosingInstanceSpecification(this.enclosingInstance, (ReferenceBinding)problemReferenceBinding1);
            return this.resolvedType;
          } 
          if (!problemReferenceBinding.isCompatibleWith((TypeBinding)targetEnclosing) && !scope.isBoxingCompatibleWith((TypeBinding)problemReferenceBinding, (TypeBinding)targetEnclosing)) {
            scope.problemReporter().typeMismatchError((TypeBinding)problemReferenceBinding, (TypeBinding)targetEnclosing, (ASTNode)this.enclosingInstance, null);
            return this.resolvedType;
          } 
          this.enclosingInstance.computeConversion((Scope)scope, (TypeBinding)targetEnclosing, (TypeBinding)problemReferenceBinding);
        } 
        if (this.arguments != null && 
          checkInvocationArguments(scope, null, (TypeBinding)anonymousSuperclass, inheritedBinding, this.arguments, this.argumentTypes, this.argsContainCast, (InvocationSite)this))
          this.bits |= 0x10000; 
        if (this.typeArguments != null && (inheritedBinding.original()).typeVariables == Binding.NO_TYPE_VARIABLES)
          scope.problemReporter().unnecessaryTypeArgumentsForMethodInvocation(inheritedBinding, this.genericTypeArguments, this.typeArguments); 
        this.binding = this.anonymousType.createDefaultConstructorWithBinding(inheritedBinding, ((this.bits & 0x10000) != 0 && this.genericTypeArguments == null));
        return this.resolvedType;
      } 
    } else if (this.enclosingInstance != null) {
      enclosingInstanceType = this.enclosingInstance.resolvedType;
      this.resolvedType = receiverType = this.type.resolvedType;
    } 
    if (isDiamond) {
      this.binding = inferConstructorOfElidedParameterizedType((Scope)scope);
      if (this.binding == null || !this.binding.isValidBinding()) {
        scope.problemReporter().cannotInferElidedTypes(this);
        return this.resolvedType = null;
      } 
      if (this.typeExpected == null && sourceLevel >= 3407872L && this.expressionContext.definesTargetType())
        return (TypeBinding)new PolyTypeBinding((Expression)this); 
      this.resolvedType = this.type.resolvedType = (TypeBinding)(referenceBinding1 = this.binding.declaringClass);
      resolvePolyExpressionArguments((Invocation)this, this.binding, this.argumentTypes, scope);
    } else {
      this.binding = findConstructorBinding(scope, (Invocation)this, referenceBinding1, this.argumentTypes);
    } 
    if (this.binding.isValidBinding()) {
      if (isMethodUseDeprecated(this.binding, (Scope)scope, true))
        scope.problemReporter().deprecatedMethod(this.binding, (ASTNode)this); 
      if (checkInvocationArguments(scope, null, (TypeBinding)referenceBinding1, this.binding, this.arguments, this.argumentTypes, this.argsContainCast, (InvocationSite)this))
        this.bits |= 0x10000; 
      if (this.typeArguments != null && (this.binding.original()).typeVariables == Binding.NO_TYPE_VARIABLES)
        scope.problemReporter().unnecessaryTypeArgumentsForMethodInvocation(this.binding, this.genericTypeArguments, this.typeArguments); 
    } else {
      if (this.binding.declaringClass == null)
        this.binding.declaringClass = referenceBinding1; 
      if (this.type != null && !this.type.resolvedType.isValidBinding())
        return null; 
      scope.problemReporter().invalidConstructor((Statement)this, this.binding);
      return this.resolvedType = (TypeBinding)referenceBinding1;
    } 
    if ((this.binding.tagBits & 0x80L) != 0L)
      scope.problemReporter().missingTypeInConstructor((ASTNode)this, this.binding); 
    if (!isDiamond && referenceBinding1.isParameterizedTypeWithActualArguments())
      checkTypeArgumentRedundancy((ParameterizedTypeBinding)referenceBinding1, scope); 
    ReferenceBinding expectedType = this.binding.declaringClass.enclosingType();
    if (TypeBinding.notEquals((TypeBinding)expectedType, enclosingInstanceType))
      scope.compilationUnitScope().recordTypeConversion((TypeBinding)expectedType, enclosingInstanceType); 
    if (enclosingInstanceType.isCompatibleWith((TypeBinding)expectedType) || scope.isBoxingCompatibleWith(enclosingInstanceType, (TypeBinding)expectedType)) {
      this.enclosingInstance.computeConversion((Scope)scope, (TypeBinding)expectedType, enclosingInstanceType);
      return this.resolvedType = (TypeBinding)referenceBinding1;
    } 
    scope.problemReporter().typeMismatchError(enclosingInstanceType, (TypeBinding)expectedType, (ASTNode)this.enclosingInstance, null);
    return this.resolvedType = (TypeBinding)referenceBinding1;
  }
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    if (visitor.visit(this, scope)) {
      if (this.enclosingInstance != null)
        this.enclosingInstance.traverse(visitor, scope); 
      if (this.typeArguments != null)
        for (int i = 0, typeArgumentsLength = this.typeArguments.length; i < typeArgumentsLength; i++)
          this.typeArguments[i].traverse(visitor, scope);  
      if (this.type != null)
        this.type.traverse(visitor, scope); 
      if (this.arguments != null) {
        int argumentsLength = this.arguments.length;
        for (int i = 0; i < argumentsLength; i++)
          this.arguments[i].traverse(visitor, scope); 
      } 
      if (this.anonymousType != null)
        this.anonymousType.traverse(visitor, scope); 
    } 
    visitor.endVisit(this, scope);
  }
}
